// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include "umfpack.h"
#include "sim.h"
#include "complex_solver.h"
#include <math.h>
#include "buffer.h"
#include <string.h>

static int last_col = 0;
static int last_nz = 0;
static double *x = NULL;
static double *xz = NULL;
static int *Ap = NULL;
static int *Ai = NULL;
static double *Ax = NULL;
static double *Az = NULL;

void complex_error_report(int status, const char *file, const char *func,
			  int line)
{
	fprintf(stderr, "in %s: file %s, line %d: ", func, file, line);

	switch (status) {
	case UMFPACK_ERROR_out_of_memory:
		fprintf(stderr, "out of memory!\n");
		break;
	case UMFPACK_WARNING_singular_matrix:
		fprintf(stderr, "matrix is singular!\n");
		break;
	default:
		fprintf(stderr, "UMFPACK error code %d\n", status);
	}
}

void complex_solver_dump_matrix(int col, int nz, int *Ti, int *Tj, double *Tx,
				double *Txz, double *b, double *bz)
{
	char build[100];
	struct buffer buf;
	buffer_init(&buf);
	buffer_malloc(&buf);
	strcpy(buf.type, "map");
	buffer_add_info(&buf);
	int i;
	for (i = 0; i < nz; i++) {
		sprintf(build, "%d %d %le\n", Tj[i], Ti[i],
			sqrt(pow(Tx[i], 2.0) + pow(Txz[i], 2.0)));
		buffer_add_string(&buf, build);

	}

	for (i = 0; i < col; i++) {
		sprintf(build, "%d %d %le\n", col, i,
			sqrt(pow(b[i], 2.0) + pow(bz[i], 2.0)));
		buffer_add_string(&buf, build);
	}

	buffer_dump("matrix.dat", &buf);

	buffer_free(&buf);
	printf("Matrix dumped\n");
}

void complex_solver_free()
{
	free(x);
	free(xz);
	free(Ap);
	free(Ai);
	free(Ax);
	free(Az);
	x = NULL;
	xz = NULL;
	Ap = NULL;
	Ai = NULL;
	Ax = NULL;
	Az = NULL;
	last_col = 0;
	last_nz = 0;
	printf("Complex solver free\n");
}

void complex_solver_print(int col, int nz, int *Ti, int *Tj, double *Tx,
			  double *Txz, double *b, double *bz)
{
	int i;
	for (i = 0; i < nz; i++) {
		printf("%d %d %le+i%le\n", Ti[i], Tj[i], Tx[i], Txz[i]);
	}

	for (i = 0; i < col; i++) {
		printf("%le+i%le\n", b[i], bz[i]);
	}

}

int complex_solver(int col, int nz, int *Ti, int *Tj, double *Tx, double *Txz,
		   double *b, double *bz)
{

	int i;
	void *Symbolic, *Numeric;
	int status;

	if ((last_col != col) || (last_nz != nz)) {
		x = realloc(x, col * sizeof(double));
		xz = realloc(xz, col * sizeof(double));
		Ap = realloc(Ap, (col + 1) * sizeof(int));
		Ai = realloc(Ai, (nz) * sizeof(int));
		Ax = realloc(Ax, (nz) * sizeof(double));
		Az = realloc(Az, (nz) * sizeof(double));
		last_col = col;
		last_nz = nz;
	}

	double Info[UMFPACK_INFO], Control[UMFPACK_CONTROL];

	umfpack_zi_defaults(Control);

	Control[UMFPACK_PRL] = 1;

	Control[UMFPACK_PRL] = 0;

	umfpack_zi_report_control(Control);

	status =
	    umfpack_zi_triplet_to_col(col, col, nz, Ti, Tj, Tx, Txz, Ap, Ai, Ax,
				      Az, NULL);

	if (status != UMFPACK_OK) {
		error_report(status, __FILE__, __func__, __LINE__);
		return EXIT_FAILURE;
	}

	status =
	    umfpack_zi_symbolic(col, col, Ap, Ai, Ax, Az, &Symbolic, Control,
				Info);
	umfpack_zi_report_status(Control, status);

	if (status != UMFPACK_OK) {
		error_report(status, __FILE__, __func__, __LINE__);
		return EXIT_FAILURE;
	}

	umfpack_zi_numeric(Ap, Ai, Ax, Az, Symbolic, &Numeric, Control, Info);

	if (status != UMFPACK_OK) {
		error_report(status, __FILE__, __func__, __LINE__);
		return EXIT_FAILURE;
	}

	status =
	    umfpack_zi_solve(UMFPACK_A, Ap, Ai, Ax, Az, x, xz, b, bz, Numeric,
			     Control, Info);
	if (status != UMFPACK_OK) {
		error_report(status, __FILE__, __func__, __LINE__);
		return EXIT_FAILURE;
	}

	(void)umfpack_zi_report_vector(col, x, xz, Control);

	umfpack_zi_free_symbolic(&Symbolic);
	umfpack_di_free_numeric(&Numeric);

	for (i = 0; i < col; i++) {
		b[i] = x[i];
		bz[i] = xz[i];
	}

	return 0;
}
