// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "epitaxy.h"
#include "inp.h"
#include "util.h"
#include "true_false.h"

void epitaxy_load(struct epitaxy *in, char *file)
{
	int i;
	char dos_file[20];
	char pl_file[20];
	struct inp_file inp;
	in->electrical_layers = 0;

	inp_init(&inp);
	inp_load(&inp, file);
	inp_check(&inp, 1.2);
	inp_reset_read(&inp);
	inp_get_string(&inp);
	sscanf(inp_get_string(&inp), "%d", &(in->layers));

	if (in->layers > 20) {
		ewe("Too many material layers\n");
	}

	if (in->layers < 1) {
		ewe("No material layers\n");
	}

	for (i = 0; i < in->layers; i++) {
		inp_get_string(&inp);
		strcpy(in->name[i], inp_get_string(&inp));
		sscanf(inp_get_string(&inp), "%le", &(in->width[i]));
		in->width[i] = fabs(in->width[i]);
		strcpy(in->mat_file[i], inp_get_string(&inp));
		strcpy(dos_file, inp_get_string(&inp));
		strcpy(pl_file, inp_get_string(&inp));

		char temp[20];
		if (strcmp(dos_file, "none") != 0) {
			strcpy(temp, dos_file);
			strcat(temp, ".inp");
			in->electrical_layer[i] = TRUE;
			if (inp_isfile(temp) != 0) {
				ewe("dos file %s does not exist", temp);
			}
			strcpy(in->dos_file[in->electrical_layers], dos_file);

			strcpy(temp, pl_file);
			strcat(temp, ".inp");
			if (inp_isfile(temp) != 0) {
				ewe("pl file %s does not exist", temp);
			}
			strcpy(in->pl_file[in->electrical_layers], pl_file);

			in->electrical_layers++;
		} else {
			in->electrical_layer[i] = FALSE;
		}

	}

	char *ver = inp_get_string(&inp);
	if (strcmp(ver, "#ver") != 0) {
		ewe("No #ver tag found in file\n");
	}

	inp_free(&inp);
}

double epitaxy_get_electrical_length(struct epitaxy *in)
{
	int i = 0;
	double tot = 0.0;

	for (i = 0; i < in->layers; i++) {
		printf("rod>>>%d\n", in->electrical_layer[i]);
		if (in->electrical_layer[i] == TRUE) {
			tot += in->width[i];
		}
	}

	return tot;
}

double epitaxy_get_optical_length(struct epitaxy *in)
{
	int i = 0;
	double tot = 0.0;

	for (i = 0; i < in->layers; i++) {
		tot += in->width[i];
	}

	return tot;
}

int epitaxy_get_optical_material_layer(struct epitaxy *in, double pos)
{
	int i = 0;
	double layer_end = 0.0;
	for (i = 0; i < in->layers; i++) {
		layer_end += in->width[i];

		if (pos < layer_end) {
			return i;
		}

	}

	return -1;
}

int epitaxy_get_electrical_material_layer(struct epitaxy *in, double pos)
{
	int i = 0;
	double layer_end = 0.0;
	int electrical_layer = 0;

	for (i = 0; i < in->layers; i++) {
		if (in->electrical_layer[i] == TRUE) {
			layer_end += in->width[i];

			if (pos < layer_end) {
				return electrical_layer;
			}
			electrical_layer++;
		}

	}

	return -1;
}

double epitaxy_get_device_start(struct epitaxy *in)
{
	int i = 0;
	double pos = 0.0;
	for (i = 0; i < in->layers; i++) {

		if (in->electrical_layer[i] == TRUE) {
			return pos;
		}
		pos += in->width[i];

	}

	return -1;
}

double epitaxy_get_device_start_i(struct epitaxy *in)
{
	int i = 0;
	for (i = 0; i < in->layers; i++) {

		if (in->electrical_layer[i] == TRUE) {
			return i;
		}

	}

	return -1;
}
